/******************************************************************************\
* $Id: reduction.sas,v 1.1.2.2.4.5 2012/07/24 11:19:22 stuhor Exp $
*
* Copyright(c) 2011 SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
*
* Name: reduction.sas
*
* Purpose: 	Keep risk factors associated with any alerts
*			Create a new alert if threshold number of risk factors is exceeded
*			Drop risk factors not associated w/ an alert
*
* Author: jabloo
*
* Support: SAS(r) Solutions OnDemand
*
* Input:	STAGE._ALERT_TDY_ONLY
*			STAGE._ALERT_RISK_FACTOR_TDY_ONLY
*
* Output: STAGE.Enriched_Alert_Summary
*
* Parameters: 	input_lib (default=STAGE)
*				output_lib (default=STAGE)
*				threshold 
*
* Dependencies/Assumptions:
*
* Usage:
*
* History:
* ddmmmyyyy userid description (Change Code)
\******************************************************************************/

%macro reduction(input_lib=STAGE, output_lib=STAGE, threshold=) ;
	%let macroname = %get_macroname;
	%put NOTE:(FCM) Entering &macroname;

	proc sort data=&output_lib.._ALERT_TDY_ONLY;
		by entity_sk entity_variable_sk;
	run;

	/* Create table of todays Fraud alerts + active Historical Fraud alerts */
	proc append base=&output_lib.._FCM_ALERT_HIST data=&output_lib.._ALERT_TDY_ONLY force nowarn;
	run;

	/* Create table for Risk Factors whose entity has an active Alert */
	proc sql;
		create table &output_lib..R_TDY as
		select 	distinct b.*
		from &output_lib.._FCM_ALERT_HIST as a, &output_lib.._ALERT_RISK_FACTOR_TDY_ONLY as b
		where 	a.entity_sk = b.entity_sk and
				a.entity_variable_sk = b.entity_variable_sk;
	quit;

	/* Create table for new Risk Factors whose entity is not associated with an active Alert */
	proc sql;
		create table To_Reduce as
		select distinct a.entity_sk, a.entity_variable_sk, a.segment_id, a.scenario_sk, a.project_sk,
				a.entity_nm, a.recipient_type_cd, a.recipient_id, a.alert_status_cd, a.alert_suppressed_flg,
				a.suppression_end_dt, a.alert_trigger_txt, a.run_dt, a.risk_flg, a.scenario_severity_score_no, a.scenario_weight_no
				/* Optional enrichment fields added 
				,a.alert_type, a.created_user_id, a.first_nm, a.middle_nm, a.last_nm, a.address_line_1_txt, a.address_line_2_txt, a.city_nm,
				a.postal_cd, a.work_phone_no, a.home_phone_no, a.gender_cd, a.birth_dt */
		from &output_lib.._ALERT_RISK_FACTOR_TDY_ONLY as a
		left join &output_lib..R_TDY as b
		on 	a.entity_sk = b.entity_sk and
			a.entity_variable_sk = b.entity_variable_sk
		where 	b.entity_sk = . and
				b.entity_variable_sk = .;
	quit;

	/* Determine if entity score > threshold.  If yes, create an alert. */
	%if %sfs_util_assert_blank_mvar(macro_variable=&threshold) %then %do; 
		%put NOTE:(FCM) Threshold for score needed to generate an alert is null. Only risk factors with an associated alert will be kept.;
	%end;
	%else %do;

		/* Determine entities whose score exceeded the threshold */
		/* Verify score exists on _alert_entity table */
		proc sql;
			create table test as
			select name from sashelp.vcolumn
			where	libname = "&input_lib" and
					memname = '_ALERT_ENTITY' and
					upcase(name) = 'OVERALL_ACTIONABLE_ENTITY_SCR';
		quit;

		/* If score exists, compare against threshold */
		%if %sfs_util_nobs(test) > 0 %then %do;
			proc sql;
				create table foo1 as
				select a.entity_sk, a.entity_variable_sk
				from &input_lib.._alert_entity as a
				where a.overall_actionable_entity_scr >= &threshold;
			quit;
		%end;
		%else 
			%put NOTE:(FCM) Score not present on _ALERT_ENTITY table.  Only risk factors with an associated alert will be kept.;  
	%end;

	%if %sfs_util_nobs(foo1) > 0 %then %do;
		/* Create an alert for those entities whose score exceeded the threshold */
		proc sql;
			create table create_risk_alert as
			select distinct a.entity_sk, a.entity_variable_sk
			from To_Reduce as a
			where 	a.entity_sk in (select entity_sk from foo1) and
					a.entity_variable_sk in (select entity_variable_sk from foo1);
		quit;

		proc sql;
			drop table foo1;
			drop table test;
		quit;

	%end;

	/* Change scenario_sk to reflect this is a newly created alert */
	%if %sfs_util_nobs(create_risk_alert) > 0 %then %do;
		
		/* Retrieve risk factors for entities that generate a new alert */
		data temp;
			drop a b; 
			merge 	create_risk_alert(in=a)		/* score > threshold */
					To_Reduce(in=b); 			
			by entity_sk entity_variable_sk;
			if a and b then
				output;
		run;

		/* Get variables from current scenario */
		proc sql noprint;
			select scenario_sk, segment_id, project_sk, recipient_id, recipient_type_cd
			into :rscenario_sk, :rsegment_id, :rproject_sk, :rrecipient_id, :rrecipient_type_cd
			from final_scenario;
		quit;

		/* Update variables for new alerts - create_risk_alert */
		data temp2;
			set create_risk_alert;
			alert_trigger_txt = "Exceeded score threshold of &threshold";
			risk_flg = 'N';
			scenario_sk = &rscenario_sk;
			segment_id = "&rsegment_id";
			project_sk = &rproject_sk;
			recipient_id = "&rrecipient_id";
			recipient_type_cd = "&rrecipient_type_cd";
			alert_suppressed_flg = 'N';
			alert_status_cd = 'ACT';
			alert_type = 'RF_SCORE';
		run;

		proc sql noprint;
			create table create_risk_alert as
			select 	distinct a.*, b.default_suppress_ms_no, b.scenario_severity_score_no, b.scenario_weight_no,
					c.entity_nm,  . as SUPPRESSION_END_DT format=datetime20.
					/* Optional Enrichment Fields added 
					,d.alert_type, d.created_user_id, d.first_nm, d.middle_nm, d.last_nm, d.address_line_1_txt, d.address_line_2_txt, d.city_nm, d.postal_cd,
					d.work_phone_no, d.home_phone_no, d.gender_cd, d.birth_dt */
			from temp2 as a
			left join FCM.FCM_SCENARIO as b
			on a.scenario_sk = b.scenario_sk
			left join FCM.FCM_ENTITY as c
			on a.entity_sk = c.entity_sk
			left join STAGE._alert_risk_factor_curr as d
			on 	a.entity_sk = d.entity_sk and
				a.entity_variable_sk = d.entity_variable_sk;
		quit;

		data create_risk_alert (drop=default_suppress_ms_no);
			set create_risk_alert;
			suppression_end_dt = datetime() + default_suppress_ms_no/1000;
		run;

		proc sql;
			drop table temp2;
		quit;

		/* Append risk factors that generated a new alert to existing risk factors */
		proc append base=&output_lib..R_TDY data=temp force nowarn;
		run;

		/* Add created alerts to the current alerts */
		proc append base=&output_lib.._ALERT_TDY_ONLY data=create_risk_alert force nowarn;
		run;
	%end;

	/* Regenerate STAGE.ENRICHED_ALERT_SUMMARY removing risk factors that did not have an associated alert */
	data &output_lib..ENRICHED_ALERT_SUMMARY;
		set	&output_lib.._ALERT_TDY_ONLY;
	run;

	proc append base=&output_lib..ENRICHED_ALERT_SUMMARY data=&output_lib..R_TDY force nowarn;
	run;

	%let macroname = %get_macroname;
	%put NOTE:(FCM) Exiting &macroname;
%mend reduction;


		
